const CodeMirror = require('./lib/all')
const {
  commonOptions,
  languageOptions
} = require('./options')

function createCodeMirror(el, options = {}) {

  const fn = el.tagName==='TEXTAREA' ? CodeMirror.fromTextArea : CodeMirror

  const {
    language = 'html',
    resizable = false,
    ...passOptions
  } = options

  const codeMirrorOptions = {
    ...commonOptions,
    ...languageOptions[language],
    ...passOptions,
    ...(
      // If no vertical resize - .CodeMirror height: auto or 100%
      !resizable || resizable==='horizontal' ?  { viewportMargin: Infinity }
        : {}
    )
  }

  let editor

  if (language==='html') Object.assign(codeMirrorOptions.extraKeys, {
    Tab: function(cm, ctx) {

      if (CodeMirror.commands.emmetExpandAbbreviation) {
        const result = CodeMirror.commands.emmetExpandAbbreviation(cm, ctx)
        if (!result) return // Returns an object if *not* expanded
      }

      const spaces = Array(editor.getOption("indentUnit") + 1).join(" ")
      editor.replaceSelection(spaces)
    },
  })

  editor = fn(el, codeMirrorOptions)

  const $editor = editor.getWrapperElement()

  if (resizable) {
    $editor.style.resize = resizable===true ? 'vertical' : resizable // horizontal, both, none
  }

  editor.element = $editor

  return editor
}

// For convenient access without calling create
createCodeMirror.CodeMirror = CodeMirror

module.exports = createCodeMirror